#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vim: shiftwidth=4 tabstop=4 expandtab softtabstop=4 ai

# Copyright 2009-2014 Odin Hørthe Omdal

# This file is part of Medlemssys.

# Medlemssys is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# Medlemssys is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with Medlemssys.  If not, see <http://www.gnu.org/licenses/>.
from __future__ import print_function

from optparse import make_option

from django.core.management.base import BaseCommand
from django.db import transaction
from django.utils import timezone

from medlemssys.medlem.models import LokallagOvervaking, Lokallag


class Command(BaseCommand):
    args = '[ delete ]'
    help = 'Opprett (og reinsk inaktive) standard LokallagsOvervakingar'
    option_list = BaseCommand.option_list + (
        make_option('-d', '--delete',
            action='store_true',
            dest='delete',
            default=False,
            help=u"""Slettar alle lokallagsovervakingar som ikkje er spesielle på nokon helst måte (altso: truleg automatisk oppretta)."""),
        )

    def handle(self, *args, **options):
        if options['delete']:
            self.delete_lokallagovervakingar()
        else:
            self.clean_stale()
            self.create_lokallagovervakingar()

    @transaction.atomic
    def clean_stale(self):
        stale = LokallagOvervaking.objects.filter(
            lokallag__aktivt=False,
            epost='',
            medlem__isnull=True,
            deaktivert__isnull=True)
        for lo in stale:
            self.out('Deactivating ' + unicode(lo))
            lo.deaktivert = timezone.now()
            lo.save()

    @transaction.atomic
    def create_lokallagovervakingar(self):
        new = Lokallag.objects.filter(aktivt=True, lokallagovervaking__isnull=True)
        for n in new:
            lo = LokallagOvervaking(lokallag=n)
            lo.save()
            self.out(u"{}: {}".format(
                    unicode(lo), unicode(lo.epostar())))

    def delete_lokallagovervakingar(self):
        """Slettar alle lokallagsovervakingar som ikkje er spesielle på nokon
        helst måte (altso: truleg automatisk oppretta)."""
        num = LokallagOvervaking.objects.filter(
            epost='',
            medlem__isnull=True).delete()
        self.out("Sletta {} overvakingar.".format(num))

    def err(self, msg):
        self.stderr.write((unicode(msg) + "\n").encode('utf-8'))
    def out(self, msg):
        self.stdout.write((unicode(msg) + "\n").encode('utf-8'))



# -*- encoding: utf-8 -*-

# -*- coding: utf-8 -*-
# Copyright 2009-2014 Odin Hørthe Omdal

# This file is part of Medlemssys.

# Medlemssys is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# Medlemssys is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with Medlemssys.  If not, see <http://www.gnu.org/licenses/>.
from __future__ import unicode_literals

import csv
import datetime
import logging
import re
import sys
import textwrap

from dateutil.parser import parse
from django.db import transaction
from reversion import revisions as reversion

from medlemssys.medlem.models import KONTI
from medlemssys.medlem.models import Medlem, Lokallag, Giro


logger = logging.getLogger(__name__)

class AccessImporter(object):
    # REGISTERKODE,LAGSNR,MEDLNR,FORNAMN,MELLOMNAMN,ETTERNAMN,TILSKRIFT1,TILSKRIFT2,POST,VERVA,VERV,LP,GJER,MERKNAD,FØDEÅR,KJØNN,INN,INNMEDL,UTB,UT_DATO,MI,MEDLEMINF,TLF_H,TLF_A,E_POST,H_TILSKR1,H_TILSKR2,H_POST,H_TLF,Ring_B,Post_B,Epost_B,MM_B,MNM_B,BRUKHEIME,FARRETOR,RETUR,REGBET,HEIMEADR,REGISTRERT,TILSKRIFT_ENDRA
    CSV_MAP = {
            'MEDLNR': 'id',
            'FORNAMN': 'fornamn',
            'MELLOMNAMN': 'mellomnamn',
            'ETTERNAMN': 'etternamn',
            'FØDEÅR': 'fodt',
            'POST': 'postnr',
            'TILSKRIFT1': 'postadr',
            'TILSKRIFT2': 'ekstraadr',
            'INN': 'innmeldt_dato',
            'UT_DATO': 'utmeldt_dato',
            'MI': 'status',
            'E_POST': 'epost',
            'TLF_H': 'mobnr',
            'TLF_A': 'heimenr',
            'LAGSNR': 'lokallag',
            'KJØNN': 'kjon',
            'GJER': 'gjer',
            'VERVA': 'innmeldingsdetalj',
            'MERKNAD': 'merknad',
        }

    VAL = (
            (29, 0, 'Ikkje ring'), # RING_B = 28 (old id)
            (30, 0, 'Ikkje post'), # POST_B = 29
            (31, 0, 'Ikkje epost'), # EPOST_B = 40
            (32, 0, 'Ikkje Motmæle'), # MM_B = 30
            (33, 0, 'Ikkje Norsk Tidend'), # MNM_B = 31
    )

    @transaction.atomic
    def import_medlem(self, medlem_fn):
        with reversion.create_revision():
            reversion.set_comment(u"CSV import")
            for num, [medlem_dict, raw_data] in enumerate(
                    self._get_medlem(medlem_fn)):
                if not self.clean_medlem_dict(medlem_dict, raw_data):
                    continue
                val = medlem_dict.get('_val', [])
                tmp = dict((k, medlem_dict[k]) for k in medlem_dict if not k.startswith('_'))
                m = Medlem(**tmp)
                m.save()

                for v in val:
                    m.set_val(v)

                # Print first 200 names
                if num < 199:
                    yield u"{0:>3}. {1}".format(unicode(num), unicode(m))

                elif num%200 == 0 and num != 0:
                    transaction.commit()
                    yield unicode(num)


    re_medlnr = re.compile(r'\[(\d+)\]')
    def clean_medlem_dict(self, medlem, raw_data=None):
        '''Cleans the member dict in-place. Returns False when unprocessed.'''

        if not medlem['id'].isdigit():
            logger.warning(u"%s(%s) ugyldig medlemsnummer! Ignorerer medlemen." % (medlem['id'], medlem['fornamn']))
            return False

        try:
            medlem['lokallag_id'] = int(medlem['lokallag'])
        except (KeyError, ValueError):
            pass
        finally:
            del medlem['lokallag']

        try:
            medlem['fodt'] = int(medlem['fodt'])
        except ValueError:
            medlem['fodt'] = None

        try:
            medlem['postnr'] = format(int(medlem['postnr']), "04d")
        except ValueError:
            logger.warning(u"%s(%s) forstår ikkje postnr: %s" % (medlem['id'], medlem['fornamn'], medlem['postnr']))
            medlem['postnr'] = "9999"

        if len(medlem.get('status', '')) > 1:
            logger.warning(u"%s(%s) status too long: %s" % (medlem['id'], medlem['fornamn'], medlem['status']))
            medlem['status'] = medlem['status'][0].upper()

        if len(medlem.get('kjon', '')) > 1:
            logger.warning(u"%s(%s) kjon too long: %s" % (medlem['id'], medlem['fornamn'], medlem['kjon']))
            medlem['kjon'] = medlem['kjon'][0].upper()

        medlem['oppretta'] = parse(medlem['innmeldt_dato'],
                default=datetime.datetime(1800, 1, 1, 0, 0))
        medlem['innmeldt_dato'] = medlem['oppretta'].date()
        medlem['oppdatert'] = datetime.datetime.now()

        if len(medlem['utmeldt_dato']) > 2:
            a = medlem['utmeldt_dato']
            medlem['utmeldt_dato'] = parse(medlem['utmeldt_dato'], default=None)
            if hasattr(medlem['utmeldt_dato'], 'date'):
                medlem['utmeldt_dato'] = medlem['utmeldt_dato'].date()
            else:
                logger.warning(u"%s(%s) utmeldt er null: %s -- org: %s" % (medlem['id'], medlem['fornamn'], medlem['utmeldt_dato'], a))
        else:
            medlem['utmeldt_dato'] = None

        # Sjekk etter [<medlemsnummer>] frå verveinfo
        if medlem['innmeldingsdetalj']:
            m = self.re_medlnr.search(medlem['innmeldingsdetalj'])
            if m:
                medlem['verva_av_id'] = m.group(1)
                medlem['innmeldingstype'] = 'D'
            elif any([x for x in ["Heimesida", "heimesida", "heimeside", "Heimeside", "heimesidene", "nettskjema"] if x in medlem['innmeldingsdetalj']]):
                medlem['innmeldingstype'] = 'H'
            elif any([x for x in [u"Målferd", u"målferd", "MF", u"målfer"] if x in medlem['innmeldingsdetalj']]):
                medlem['innmeldingstype'] = 'M'
            elif any([x for x in ["Flygeblad", "flygeblad", "flygis", "flogskrift", "Flogskrift", "Opprop", "opprop"] if x in medlem['innmeldingsdetalj']]):
                medlem['innmeldingstype'] = 'F'
            elif any([x for x in ["SMS", "sms"] if x in medlem['innmeldingsdetalj']]):
                medlem['innmeldingstype'] = 'S'
            elif any([x for x in ["Lagsskiping", "lagsskiping", u"årsmøte", u"Årsmøte"] if x in medlem['innmeldingsdetalj']]):
                medlem['innmeldingstype'] = 'L'
            elif any([x for x in ["Vitjing", "leir", "Leir"] if x in medlem['innmeldingsdetalj']]):
                medlem['innmeldingstype'] = 'O'
            elif any([x for x in ["Telefon", "telefon", "tlf"] if x in medlem['innmeldingsdetalj']]):
                medlem['innmeldingstype'] = 'A'

        medlem['_val'] = []
        for v in self.VAL:
            if raw_data[v[0]] != "" and int(raw_data[v[0]]) == int(v[1]):
                medlem['_val'].append(v[2])

        return True


    def _get_medlem(self, medlem_fn):
        liste = csv.reader(open(medlem_fn))
        mapping = self._create_mapping(headers=liste.next())
        if not mapping:
            logger.error(u"%s: ser ikkje ut som ein medlems-csv fil (manglar header?)" % self.medlem_fil)
            sys.exit()
        for rad in liste:
            tmp = {}
            for typ in self.CSV_MAP.values():
                if typ not in mapping:
                    logger.warning(u"Type '" + unicode(typ) + u"' not in mapping. row = " + unicode(rad))
                tmp[typ] = rad[mapping[typ]] \
                            .decode("utf-8") \
                            .replace(r"\n", "\n")
            yield tmp, rad


    def _create_mapping(self, headers):
        mapping = dict()
        for h in self.CSV_MAP.keys():
            if h in headers:
                mapping[self.CSV_MAP[h]] = headers.index(h)
            else:
                raise Exception("Fann ikkje {0} som header.".format(h))
        return mapping


    # csv: DIST,FLAG,LLAG,lid,ANDSVAR,LOKALSATS
    # model: namn, fylkeslag, distrikt, andsvar
    @transaction.atomic
    @reversion.create_revision()
    def import_lag(self):
        reversion.set_comment("Import frå Access")
        liste = csv.reader(open(self.lag_fil))
        liste.next()
        alle_lag = {}

        for rad in liste:
            namn = rad[2].decode('utf-8')       \
                    .title()                    \
                    .replace(' Og ', ' og ')    \
                    .replace(' I ', ' i ')      \
                    .replace(u' På ', u' på ')  \
                    .replace('Fmu', 'FMU')      \
                    .replace('Nmu', 'NMU')      \
                    .replace(' Mu', ' MU')

            lag = Lokallag.objects.filter(pk=rad[3])
            if len(lag):
                lag = lag[0]
                lag.namn = namn
                lag.andsvar = rad[4].decode('utf-8')
            else:
                lag = Lokallag(pk=rad[3], namn=namn, andsvar=rad[4].decode('utf-8'))
            lag.save()

            alle_lag[rad[3]] = lag

        return alle_lag

    # csv: 0:MEDLNR, 1:BETALT, 2:KONTO, 3:KONTONAMN, 4;DATO, 5:AR, 6:SUM, 7:BETID
    # model (giro): medlem, belop, kid, oppretta, innbetalt, konto, hensikt, desc
    @transaction.atomic
    @reversion.create_revision()
    def import_bet(self, force_update):
        liste = csv.reader(open(self.bet_fil))
        liste.next()
        try:
            highest_pk = Giro.objects.all().order_by("-pk")[0].pk
        except:
            highest_pk = -1

        for num, rad in enumerate(liste):
            if num % 1000 == 0 and num != 0:
                transaction.commit()
                yield unicode(num)

            # Hopp over ferdig-importerte betalingar
            if not force_update and int(rad[7]) < highest_pk:
                continue

            if not force_update and Giro.objects.filter(pk=rad[7]).exists():
                continue

            g = Giro(pk=rad[7], hensikt='P', status='F')

            # Finn andsvarleg medlem
            try:
                g.medlem = Medlem.objects.alle().get(pk=rad[0])
            except Medlem.DoesNotExist:
                logger.warning(u"Fann ikkje medlem %s. %s " % (rad[0], rad))
                continue

            # Kva konto
            if rad[2].upper() in [ k[0] for k in KONTI ]:
                g.konto = rad[2].upper()
            elif rad[1].upper() == 'L':
                # Livstidsmedlem
                if not g.medlem.status == 'L':
                    g.medlem.status = 'L'
                    g.medlem.save()
                    logger.warning(u"Oppdaterte %s til L. %s " % (g.medlem, rad))
                logger.warning(u"Hopper over %s (L). %s " % (g.medlem, rad))
                continue
            elif rad[6].isdigit():
                # Viss det er pengar gjeng me ut i frå at det er snakk om kasse
                g.konto = 'K'
                logger.warning(u"Gjettar at pengar (%s) er til KASSE %s -- %s" % (rad[6], g.medlem, rad))
            else:
                logger.warning(u"Kjenner ikkje att konto: {0}, {1}".format(g.medlem, rad))
                continue

            # Kor mykje pengar inn?
            try:
                g.belop = float(rad[6])
            except ValueError:
                g.belop = 0
                logger.warning(u"Beløp = 0: {0}, {1}".format(g.medlem, rad))

            if len(rad[4]) > 3:
                g.innbetalt = parse(rad[4])
                g.innbetalt_belop = g.belop
            else:
                logger.warning(u"Ikkje innbetalt! {0}, {1}".format(g.medlem, rad))

            try:
                g.oppretta = datetime.datetime(int(rad[5]), 1, 1, 0, 0)
            except ValueError:
                logger.warning(u"Gjettar oppretta==innbetalt: {0}, {1}".format(g.medlem, rad))
                g.oppretta = g.innbetalt

            g.gjeldande_aar = g.oppretta.year

            if rad[3] != "MEDLEMSKONTO":
                g.desc = rad[3]

            g.save()

            if num < 99:
                yield unicode(g)

class MamutImporter(AccessImporter):
    CSV_MAP = {
            'cont': '_is_customer',
            'contid': 'id',
            'name': 'fornamn',
            'zipcode': 'postnr',
            'street': 'postadr',
            'city': 'ekstraadr',
            'regdate': 'innmeldt_dato',
            'editdate': 'oppdatert',
            'active': '_active',
            'email': 'epost',
            'phone1': 'mobnr',
            'gruppe': '_gruppe',
        }

    def clean_medlem_dict(self, medlem, raw_data=None):
        if not medlem['id'].isdigit():
            logger.error(u"%s(%s) ugyldig medlemsnummer! Ignorerer medlemen." % (medlem['id'], medlem['fornamn']))
            return False
        if medlem['_is_customer'] != "SANN":
            logger.info(u"%s(%s) ikkje customer. Ignorerer medlemen." % (medlem['id'], medlem['fornamn']))
            return False

        namn = medlem['fornamn'].split()
        if len(namn) > 1:
            medlem['fornamn'] = " ".join(namn[:-1])
            medlem['etternamn'] = namn[-1]
        else:
            medlem['etternamn'] = '-'

        medlem['oppretta'] = parse(medlem['innmeldt_dato'],
                default=datetime.datetime(1800, 1, 1, 0, 0))
        medlem['innmeldt_dato'] = medlem['oppretta'].date()
        medlem['oppdatert'] = parse(medlem['oppdatert'],
                default=datetime.datetime(1800, 1, 1, 0, 0))


        if not '@' in medlem['epost']:
            medlem['merknad'] = medlem['epost']
            medlem['epost'] = ''
        elif ' ' in medlem['epost']:
            raise Exception("Mellomrom i epost")

        if medlem['mobnr']:
            medlem['mobnr'] = ''.join(medlem['mobnr'].split())
            if len(medlem['mobnr']) < 4:
                del medlem['mobnr']

        grupper = medlem['_gruppe'].splitlines()
        if medlem['_active'] != "SANN":
            medlem['utmeldt_dato'] = medlem['oppdatert'].date()
        elif 'sagt opp' in medlem['_gruppe'].lower():
            sagtopp = next((re.match('Sagt opp (\d+)', x)
                           for x in grupper if 'Sagt opp' in x))
            if not sagtopp:
                raise Exception("wat! nutin: " + unicode(grupper))
            medlem['utmeldt_dato'] = datetime.date(int(sagtopp.group(1)), 1, 1)
            grupper.remove('Sagt opp ' + sagtopp.group(1))

        if any(k in medlem['_gruppe'].lower()
               for k in ('utmeldt', 'fjerna', 'sagt opp')):
            if 'utmeldt_dato' not in medlem:
                logger.info(u"%s(%s) utmeldt, men aktiv. gruppe:%s." % (medlem['id'], medlem['fornamn'], medlem['_gruppe'][:-1]))
        elif ('utmeldt_dato' in medlem and
                any(k in medlem['_gruppe'].lower() for k in ('tingar'))):
            logger.error(u"%s(%s %s) FEILUTMELDING. gruppe:%s." % (medlem['id'], medlem['fornamn'], medlem['etternamn'], medlem['_gruppe'][:-1]))
            #raise Exception("utmeldt utan å vera det.")

        lokallag=None
        for g in grupper:
            if g.startswith('Tingar:') or g.startswith('Gruppetingarar:'):
                lokallag = g
                grupper.remove(lokallag)
                break
        # Gjer resterande til val
        medlem['_val'] = grupper

        if lokallag:
            medlem['lokallag'] = Lokallag.objects.get_or_create(
                namn=lokallag.strip(),
                defaults={ 'fylkeslag': '', 'distrikt': '', 'andsvar': ''})[0]

        return True

class GuessingCSVImporter(AccessImporter):
    MATCH_LOOKUP = {
      'id': ['pk', 'nr', 'medlemsnr', 'medlnr', 'medlid', 'medlemsid',
             'mid', 'mnr', 'nummer'],
      'fornamn': ['fornavn', 'firstname', 'namn', 'navn'],
      'mellomnamn': ['mellomnavn', 'middlename'],
      'etternamn': ['etternavn', 'lastname'],
      'fodt': ['født', 'år', 'born', 'year', 'fødeår'],
      'postnr': ['postnummer', 'zip', 'zipcode', 'post'],
      'epost': ['e-post', 'email'],
      'postadr': ['postadresse', 'postadresse1', 'heimetilskrift',
                  'heimeadresse', 'hjemmeadresse', 'heaimeadr', 'hjemmeadr',
                  'htilskrift', 'hadresse', 'htilskr', 'gateadresse',
                  'gateadresse1', 'addresse', 'tilskrift', 'tilskr', 'adr'],
      'mobnr': ['mobil', 'tlfmobil', 'tlfnr', 'mobilnummer', 'telefonnummer'],
      'merknad': ['notat', 'kommentar', 'notes', 'comment', 'extra', 'ekstra'],
      'kjon': ['kjøn', 'kjønn', 'kjonn', 'sex'],
      'gjer': ['gjør', 'workplace', 'what'],
      'innmeldt_dato': ['innmeldttid', 'innmeldt'],
      'utmeldt_dato': ['utmeldttid', 'utmeldt', 'utdato', 'datout'],
      'oppretta': ['created', 'creationdate', 'creationtime',],
      'oppdatert': ['updated', 'updatedate', 'lastupdated', 'oppdaterttid', 'oppdatertdato'],
      '_lokallag': ['lokallag', 'lag', 'gruppe'],
      '_val': ['val', 'valg', 'grupper'],
      'heimenr': ['tlfheime', 'tlfhjemme', 'tlfprivat', 'privattlf'],
      'bortepostnr': [],
      'borteadr': [],
      'status': [],
      'mellomnamn': [],
    }

    REQUIRED = ['id', 'fornamn', 'fodt', 'postnr']

    def __init__(self, *args, **kwargs):
        self._matches = {v: None for v in self.REQUIRED}
        super(GuessingCSVImporter, self).__init__(*args, **kwargs)

    def _get_medlem(self, medlem_fn):
        fp = open(medlem_fn)
        dialect = csv.Sniffer().sniff(fp.read(1024), delimiters=str(';,'))
        fp.seek(0)
        reader = csv.DictReader(open(medlem_fn), dialect=dialect)
        fields = [f.decode('utf-8') for f in reader.fieldnames]
        if len(fields) < 5:
            print(textwrap.dedent("""
                Suspicously few fields: {entr}.
                Wrong delimiter?  Headers not on first line?

                Quitting.
                """).format(entr=', '.join(unicode(s) for s in fields)))
            sys.exit(1)

        self._create_mapping(fields, self._matches)
        missing_fields = (
            set(self.MATCH_LOOKUP.keys()) - set(self._matches.keys()))
        unused_fields = (
            set(fields) -
            set(f.decode('utf-8') for f in self._matches.values() if f))
        if missing_fields or unused_fields:
            print(textwrap.dedent("""
                The fields and database don't match up perfectly.

                Fields in database that's missing a mapping from CSV:
                {missing}

                Fields in CSV that's unused for a mapping to database:
                {unused}
                """).format(
                    missing='\n'.join(sorted(missing_fields)),
                    unused='\n'.join(sorted(unused_fields)))
                )
        print(textwrap.dedent("""
            The mapping between CSV and database is:
              {mapping}
            """).format(mapping = self._matches))
        missing_required = [k for k, v in self._matches.items() if not v]
        if missing_required:
            print(textwrap.dedent("""
                Some required fields are missing:
                  {}
                """).format(', '.join(missing_required)))
            sys.exit(1)
        raw_input('Ctrl+C to stop. Enter to continue.');

        for line in reader:
            tmp = {db_key: line[csv_key].decode('utf-8') for
                       db_key, csv_key in self._matches.items()}
            yield tmp, line

    def _create_mapping(self, input_keys, matches):
        def find_matches(key_normalizer):
            for original_key in input_keys:
                normalized_key = key_normalizer(original_key)
                for match_key, match_values in self.MATCH_LOOKUP.items():
                    if matches.get(match_key):
                        continue
                    for match_variation in [match_key] + match_values:
                        if normalized_key == match_variation:
                            # csv module in Python 2 is totally fucked up.
                            # It deals with bytes-as-strings and not unicode
                            # So this is messed up.
                            matches[match_key] = original_key.encode('utf-8')
                            break
        find_matches(lambda k: ''.join(k.lower().split()))
        find_matches(lambda k: re.sub(r'[^\w]', '',  k.lower()))
        find_matches(lambda k: re.sub(r'[\W\d_-]', '',  k.lower()))

    def clean_medlem_dict(self, medlem, raw_data=None):
        if not medlem['id'].isdigit():
            logger.error(u"%s(%s) ugyldig medlemsnummer! Ignorerer medlemen." %
                (medlem['id'], medlem['fornamn']))
            return False

        if 'fornamn' in medlem and not 'etternamn' in medlem:
            namn = medlem['fornamn'].split()
            if len(namn) > 1:
                medlem['fornamn'] = " ".join(namn[:-1])
                medlem['etternamn'] = namn[-1]
            else:
                medlem['etternamn'] = '-'

        try:
            medlem['fodt'] = int(medlem['fodt'])
        except ValueError:
            medlem['fodt'] = None

        if len(medlem.get('kjon', '')) > 1:
            kjon = medlem['kjon'].lower()
            if kjon in ['g', 'gutt', 'm', 'mann', 'male', 'man']:
                medlem['kjon'] = 'M'
            elif kjon in ['j', 'jente', 'k', 'kvinne', 'female', 'woman']:
                medlem['kjon'] = 'K'
            else:
                logger.warning(u"%s(%s) dunno kjon: %s" % (medlem['id'], medlem['fornamn'], medlem['kjon']))
                medlem['kjon'] = 'U'

        if (medlem['postnr'].isdigit() and
                medlem['postnr'] > '0000' and
                medlem['postnr'] < '9999'):
            medlem['ekstraadr'] = ''
        else:
            medlem['ekstraadr'] = ' '.join((medlem['postnr'],
                                            medlem.get('ekstraadr', '')))
            medlem['postnr'] = '9999'

        fake_date = datetime.datetime(1800, 1, 1, 0, 0)
        def makedate(key, fallback=fake_date):
            try:
                return parse(medlem[key], default=fallback)
            except:
                return fallback

        if not 'oppretta' in medlem:
            medlem['oppretta'] = makedate('innmeldt_dato')
        medlem['oppdatert'] =  makedate('oppdatert', datetime.datetime.now())
        medlem['innmeldt_dato'] = makedate('innmeldt_dato')
        medlem['utmeldt_dato'] =  makedate('utmeldt_dato', None)

        if medlem.get('epost'):
            if not '@' in medlem['epost']:
                medlem['merknad'] = medlem['epost']
                medlem['epost'] = ''
            if '<' in medlem['epost']:
                m = re.search(r'<([^>]+)>', medlem['epost'])
                if m and m.group(1):
                    medlem['epost'] = m.group(1)
            if ' ' in medlem['epost']:
                raise Exception("Mellomrom i epost")

        if medlem.get('mobnr'):
            medlem['mobnr'] = ''.join(medlem['mobnr'].split())
            if len(medlem['mobnr']) < 4:
                del medlem['mobnr']

        if medlem.get('_lokallag'):
            medlem['lokallag'] = Lokallag.objects.get_or_create(
                namn=medlem['_lokallag'].strip(),
                defaults={ 'fylke': '', 'kommunes': '', 'andsvar': ''})[0]

        return True



# -*- coding: utf-8 -*-
# Copyright 2009-2016 Odin Hørthe Omdal

# This file is part of Medlemssys.

# Medlemssys is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# Medlemssys is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with Medlemssys.  If not, see <http://www.gnu.org/licenses/>.
import django.views.static
from django.conf import settings
from django.conf.urls import include
from django.conf.urls import url
from django.contrib import admin
from django.views.generic.base import TemplateView

import giro.admin_views
import innhenting.views
import medlem.lokallag
import medlem.views
import statistikk.admin_views
import statistikk.views


admin.autodiscover()

urlpatterns = [
]

if settings.DEBUG:
    urlpatterns += [
        url(r'^static/(?P<path>.*)$', django.views.static.serve,
            {'document_root': '/home/odin/Kode/medlemssys/static'}),
    ]



# -*- coding: utf-8 -*-
"""
Production Configurations

You need to go through this file and change the settings.

"""
from __future__ import absolute_import, unicode_literals

import os

from medlemssys.config.settings import *  # noqa


DEBUG = True
if DEBUG:
    DATABASES = {
        'default': {
            'ENGINE': 'django.db.backends.sqlite3',
            'NAME': os.path.join(os.path.normpath(os.path.dirname(__file__)),
                                 'db.sqlite'),
        }
    }

# Hosts/domain names that are valid for this site
ALLOWED_HOSTS = []

# Make this unique, and don't share it with anybody.
SECRET_KEY = 'ENDRA_MEG-e!(o%l1myqy-v(ocxf*xkr)q#=l-^%yxgcod_uicne1wh5ggi1'

# Token used for creating medlems via API. To stop stupid bots.
INNMELDING_TOKEN = ''

# Used for allowing CORS, and where the ?next= redirector can point
TRUSTED_EXTERNAL_DOMAINS = []

# This can be used to implement specific behaviours [barnogungdom, base]
BEHAVIOUR_MODULE = 'medlemssys.behaviour.barnogungdom'

# Installation specific data
KONTONUMMER = '3450 65 48618'
ORGNUMMER = '959 358 451'

# Email
DEFAULT_FROM_EMAIL = 'Medlemssys <noreply@medlemssys.s0.no>'
EMAIL_SUBJECT_PREFIX = '[Medlemssys] '
SERVER_EMAIL = DEFAULT_FROM_EMAIL

# Database configuration
# See: https://docs.djangoproject.com/en/dev/ref/settings/#databases
#DATABASES = {
#    'default': {
#        'ENGINE': 'django.db.backends.postgresql',
#        'NAME': 'medlemssys',
#        'USER': '',
#        'PASSWORD': '',
#    }
#}

#
# Less frequently changed settings
#

# Custom Admin URL
#ADMIN_URL = r'^admin/'

# Use Whitenoise to serve static files
# See: https://whitenoise.readthedocs.io/
#STATIC_ROOT = ''
MIDDLEWARE_CLASSES = (
    'whitenoise.middleware.WhiteNoiseMiddleware',) + MIDDLEWARE_CLASSES
STATICFILES_STORAGE = 'whitenoise.storage.CompressedManifestStaticFilesStorage'

# SECURITY CONFIGURATION
# See https://docs.djangoproject.com/en/1.9/ref/middleware/#module-django.middleware.security
# and https://docs.djangoproject.com/ja/1.9/howto/deployment/checklist/#run-manage-py-check-deploy

# set this to 60 seconds and then to 518400 when you can prove it works
#SECURE_HSTS_SECONDS = 60
#SECURE_HSTS_INCLUDE_SUBDOMAINS = True
#SECURE_CONTENT_TYPE_NOSNIFF = True
#SECURE_BROWSER_XSS_FILTER = True
#SESSION_COOKIE_SECURE = True
#SESSION_COOKIE_HTTPONLY = True
#SECURE_SSL_REDIRECT = True
#CSRF_COOKIE_SECURE = True
#CSRF_COOKIE_HTTPONLY = True
#X_FRAME_OPTIONS = 'DENY'

if not DEBUG:
    INSTALLED_APPS += ('gunicorn', )


# Template configuration
# See:
# https://docs.djangoproject.com/en/dev/ref/templates/api/#django.template.loaders.cached.Loader
TEMPLATES[0]['OPTIONS']['loaders'] = [
    ('django.template.loaders.cached.Loader', [
         'django.template.loaders.filesystem.Loader',
         'django.template.loaders.app_directories.Loader',
     ]),
]

# Caching
# Comment out to enable, you obviously need Redis running.
"""
CACHES = {
    'default': {
        'BACKEND': 'django_redis.cache.RedisCache',
        'LOCATION': 'redis://127.0.0.1:6379/0',
        'OPTIONS': {
            'CLIENT_CLASS': 'django_redis.client.DefaultClient',
            'IGNORE_EXCEPTIONS': True,  # mimics memcache behavior.
                                        # http://niwinz.github.io/django-redis/latest/#_memcached_exceptions_behavior
        }
    }
}
"""


# Raven Sentry client
# This is currently disabled. You need to install `raven` and comment out this
# configuration.
# See https://docs.getsentry.com/hosted/clients/python/integrations/django/
"""
INSTALLED_APPS += ('raven.contrib.django.raven_compat', )
MIDDLEWARE_CLASSES = (
    'raven.contrib.django.raven_compat.middleware.'
    'SentryResponseErrorIdMiddleware',) + MIDDLEWARE_CLASSES

# Sentry Configuration
SENTRY_DSN = 'your-dsn'
SENTRY_CLIENT = 'raven.contrib.django.raven_compat.DjangoClient'
LOGGING = {
    'version': 1,
    'disable_existing_loggers': True,
    'root': {
        'level': 'WARNING',
        'handlers': ['sentry'],
    },
    'formatters': {
        'verbose': {
            'format': '%(levelname)s %(asctime)s %(module)s '
                      '%(process)d %(thread)d %(message)s'
        },
    },
    'handlers': {
        'sentry': {
            'level': 'ERROR',
            'class': 'raven.contrib.django.raven_compat.handlers.SentryHandler',
        },
        'console': {
            'level': 'DEBUG',
            'class': 'logging.StreamHandler',
            'formatter': 'verbose'
        }
    },
    'loggers': {
        'django.db.backends': {
            'level': 'ERROR',
            'handlers': ['console'],
            'propagate': False,
        },
        'raven': {
            'level': 'DEBUG',
            'handlers': ['console'],
            'propagate': False,
        },
        'sentry.errors': {
            'level': 'DEBUG',
            'handlers': ['console'],
            'propagate': False,
        },
        'django.security.DisallowedHost': {
            'level': 'ERROR',
            'handlers': ['console', 'sentry'],
            'propagate': False,
        },
    },
}
RAVEN_CONFIG = {
    'DSN': SENTRY_DSN
}
"""
# Raven end



# -*- coding: utf-8 -*-
# Copyright 2009-2016 Odin Hørthe Omdal

# This file is part of Medlemssys.

# Medlemssys is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# Medlemssys is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with Medlemssys.  If not, see <http://www.gnu.org/licenses/>.
from __future__ import unicode_literals

import django.contrib.auth.views
from django.conf import settings
from django.conf.urls import include, url
from django.conf.urls.static import static
from django.contrib import admin
from django.views.generic import TemplateView
from django.views import defaults as default_views

import medlemssys.api.urls as api_urls
import medlemssys.giro.admin_views as giro_admin
import medlemssys.innhenting.views as innhenting
import medlemssys.medlem.lokallag as lokallag
import medlemssys.medlem.views as medlem
import medlemssys.statistikk.admin_views as statistikk_admin
import medlemssys.statistikk.views as statistikk


urlpatterns = [
    url(r'^$',
        TemplateView.as_view(template_name='pages/home.html'), name='home'),
    url(r'^robots.txt$',
        TemplateView.as_view(template_name='pages/robots.txt'), name='robots'),

    url(r'^admin/innhenting/import_ocr/',
        innhenting.import_ocr, name='import_ocr'),
    url(r'^admin/medlem/giro/send/', giro_admin.send, name='giro_send'),
    url(r'^admin/medlem/giro/manual/',
        giro_admin.manual_girosearch, name='giro_manual'),
    url(r'^admin/revisions/',
        statistikk_admin.show_revisions, name='show_revisions'),
    # Django Admin, use {% url 'admin:index' %}
    url(settings.ADMIN_URL, include(admin.site.urls)),

    url(r'^accounts/login/$',
        django.contrib.auth.views.login, { 'template_name': 'login.html' }),
    url(r'^accounts/logout/$',
        django.contrib.auth.views.logout, { 'next_page': '/' }),
    url(r'^medlem/opprett', medlem.create_medlem),
    url(r'^medlem/(?P<id>\d+)/endra/(?P<nykel>\w+)',
        medlem.edit_medlem, name='medlem_edit'),
    url(r'^medlem/ringjelister', medlem.ringjelister),
    url(r'^lokallag/$', lokallag.home),
    url(r'^lokallag/(?P<slug>[-\w]+)/$',
        lokallag.lokallag_home, name='lokallag_home'),
    url(r'^takk', TemplateView.as_view(template_name='takk.html')),
    url(r'^stats/vervetopp/', statistikk.vervetopp),
    url(r'^stats/vervometer/', statistikk.vervometer),
    url(r'^member/search/', medlem.search, name='search'),
    url(r'^api/get_members.json',
        medlem.get_members_json, name='get_members_json'),
    url(r'^api/', include(api_urls), name='api-root'),

] + static(settings.MEDIA_URL, document_root=settings.MEDIA_ROOT)

if settings.DEBUG:
    # This allows the error pages to be debugged during development, just visit
    # these url in browser to see how these error pages look like.
    urlpatterns += [
        url(r'^400/$', default_views.bad_request,
            kwargs={'exception': Exception('Bad Request!')}),
        url(r'^403/$', default_views.permission_denied,
            kwargs={'exception': Exception('Permission Denied')}),
        url(r'^404/$', default_views.page_not_found,
            kwargs={'exception': Exception('Page not Found')}),
        url(r'^500/$', default_views.server_error),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9.7 on 2016-06-30 21:16
from __future__ import unicode_literals

import datetime
from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import medlemssys.medlem.models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Giro',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('belop', models.PositiveIntegerField(verbose_name='Bel\xf8p')),
                ('innbetalt_belop', models.PositiveIntegerField(default=0, verbose_name='Innbetalt bel\xf8p')),
                ('kid', models.CharField(blank=True, max_length=255, unique=True, verbose_name='KID-nummer')),
                ('oppretta', models.DateTimeField(auto_now_add=True, verbose_name='oppretta')),
                ('oppdatert', models.DateTimeField(auto_now=True, verbose_name='oppatert')),
                ('innbetalt', models.DateField(blank=True, null=True, verbose_name='Dato betalt')),
                ('konto', models.CharField(choices=[('A', 'Medlemskonto (KID)'), ('M', 'Medlemskonto'), ('K', 'Kassa'), ('B', 'Brukskonto'), ('S', 'SMS')], default='M', max_length=1, verbose_name='Konto')),
                ('hensikt', models.CharField(choices=[('P', 'Medlemspengar'), ('G', 'G\xe5ve'), ('T', 'Tilskiping'), ('A', 'Anna')], default='P', max_length=1, verbose_name='Hensikt')),
                ('desc', models.TextField(blank=True, default='', verbose_name='Forklaring')),
                ('status', models.CharField(choices=[('V', 'Ventar'), ('1', 'Epost sendt'), ('E', '(E) Sendingsfeil'), ('M', 'Manuelt postlagt'), ('F', 'Ferdig'), ('U', 'Utg\xe5tt ubetalt')], default='V', max_length=1, verbose_name='Status')),
                ('gjeldande_aar', models.PositiveIntegerField(default=medlemssys.medlem.models.this_year, verbose_name='Gjeldande \xe5r')),
            ],
            options={
                'ordering': ('-oppdatert', '-innbetalt', '-pk'),
                'verbose_name_plural': 'giroar',
            },
        ),
        migrations.CreateModel(
            name='Innmeldingstype',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('namn', models.CharField(max_length=255, unique=True, verbose_name='namn')),
            ],
            options={
                'verbose_name_plural': 'innmeldingstypar',
            },
        ),
        migrations.CreateModel(
            name='Lokallag',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('namn', models.CharField(max_length=255, unique=True, verbose_name='namn')),
                ('slug', models.SlugField(blank=True, max_length=255, unique=True, verbose_name='nettnamn')),
                ('kommunes', models.CharField(blank=True, help_text='Kommaseparert med stor forbokstav, t.d. "Stavanger, Sandnes".', max_length=255, verbose_name='kommunar')),
                ('fylke', models.CharField(blank=True, help_text='Berre eitt fylke, stor forbokstav. Overstyrer kommuner visst denne finst.', max_length=255, verbose_name='fylke')),
                ('andsvar', models.CharField(blank=True, max_length=255, verbose_name='andsvar')),
                ('lokalsats', models.IntegerField(default=0, verbose_name='lokalsats')),
                ('aktivt', models.BooleanField(default=True, verbose_name='aktivt')),
                ('epost', models.EmailField(blank=True, help_text='Eiga epostliste til styret. Tek prioritet\n        over \xe5 senda enkeltvis til dei som er registrert som styre.', max_length=254, verbose_name='epost')),
            ],
            options={
                'ordering': ['-aktivt', 'namn'],
                'verbose_name_plural': 'lokallag',
            },
        ),
        migrations.CreateModel(
            name='LokallagOvervaking',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('epost', models.CharField(blank=True, help_text='Vert brukt dersom medlem ikkje er satt', max_length=255, verbose_name='epost')),
                ('deaktivert', models.DateTimeField(blank=True, null=True)),
                ('sist_oppdatert', models.DateTimeField(auto_now=True)),
                ('lokallag', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='lokallag_overvaking_set', to='medlem.Lokallag')),
            ],
            options={
                'verbose_name_plural': 'lokallagovervakingar',
            },
        ),
        migrations.CreateModel(
            name='Medlem',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('fornamn', models.CharField(max_length=255, verbose_name='fornamn')),
                ('mellomnamn', models.CharField(blank=True, max_length=255, verbose_name='mellomnamn')),
                ('etternamn', models.CharField(max_length=255, verbose_name='etternamn')),
                ('fodt', models.IntegerField(blank=True, null=True, verbose_name='f\xf8dt')),
                ('postnr', models.CharField(help_text="'0000' viss norsk adresse manglar", max_length=4, verbose_name='postnr')),
                ('epost', models.CharField(blank=True, max_length=255, verbose_name='epost')),
                ('postadr', models.CharField(blank=True, help_text='Skriv utanlands-/skuleadresse i borte-addresse', max_length=255, null=True, verbose_name='postadresse')),
                ('ekstraadr', models.CharField(blank=True, max_length=255, verbose_name='ekstraadresse')),
                ('mobnr', models.CharField(blank=True, max_length=50, verbose_name='mobiltelefon')),
                ('heimenr', models.CharField(blank=True, max_length=50, verbose_name='heimetelefon')),
                ('bortepostnr', models.CharField(blank=True, help_text="T.d. '0864' eller 'c/o John Smith, 50012 OX-5, Oxford England'", max_length=255, null=True, verbose_name='borte-postnr')),
                ('borteadr', models.CharField(blank=True, help_text='Dersom borte-adressene er sett, vert post sendt her. For t.d. hybel- og utenlandsadresser.', max_length=255, null=True, verbose_name='borte-adresse')),
                ('gjer', models.CharField(blank=True, max_length=255, verbose_name='gjer')),
                ('kjon', models.CharField(choices=[('M', 'Mann'), ('K', 'Kvinne'), ('U', 'Udefinert')], default='U', max_length=1, verbose_name='kj\xf8nn')),
                ('merknad', models.TextField(blank=True, default='', verbose_name='merknad')),
                ('_siste_medlemspengar', models.PositiveIntegerField(blank=True, default=None, editable=False, null=True)),
                ('innmeldt_dato', models.DateField(default=datetime.date.today, verbose_name='innmeldt')),
                ('utmeldt_dato', models.DateField(blank=True, default=None, null=True, verbose_name='utmeldt')),
                ('status', models.CharField(choices=[('M', 'Vanleg medlem'), ('I', 'Infoperson'), ('L', 'Livstidsmedlem')], default='M', max_length=1, verbose_name='medlstatus')),
                ('innmeldingstype', models.CharField(choices=[('H', 'Heimesida'), ('M', 'M\xe5lferd'), ('V', 'Vervekampanje'), ('F', 'Flygeblad'), ('L', 'Lagsskiping/\xe5rsm\xf8te'), ('D', 'Direkteverva'), ('S', 'SMS'), ('O', 'Vitjing'), ('A', 'Anna'), ('U', '--')], default='U', max_length=1, verbose_name='innmtype')),
                ('innmeldingsdetalj', models.CharField(blank=True, help_text='Skriv inn vervemedlem i hakeparantesar ([1234])', max_length=255, verbose_name='detalj om innmelding')),
                ('oppretta', models.DateTimeField(auto_now_add=True, verbose_name='oppretta')),
                ('oppdatert', models.DateTimeField(auto_now=True, verbose_name='oppdatert')),
                ('nykel', models.CharField(blank=True, default=medlemssys.medlem.models.generate_password, max_length=255, verbose_name='nykel')),
                ('betalt_av', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='betalar_for', to='medlem.Medlem')),
                ('lokallag', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='medlem.Lokallag')),
            ],
            options={
                'ordering': ['-id'],
                'verbose_name_plural': 'medlem',
            },
        ),
        migrations.CreateModel(
            name='PostNummer',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('postnr', models.CharField(max_length=6)),
                ('poststad', models.CharField(max_length=50)),
                ('bruksomrade', models.CharField(max_length=50)),
                ('folketal', models.SmallIntegerField(blank=True, null=True)),
                ('bydel', models.CharField(max_length=50)),
                ('kommnr', models.CharField(max_length=50)),
                ('kommune', models.CharField(max_length=50)),
                ('fylke', models.CharField(max_length=50)),
                ('lat', models.FloatField()),
                ('lon', models.FloatField()),
                ('datakvalitet', models.SmallIntegerField()),
                ('sist_oppdatert', models.DateTimeField(blank=True, null=True)),
            ],
            options={
                'verbose_name_plural': 'postnummer',
            },
        ),
        migrations.CreateModel(
            name='Rolle',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('lokallag', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='rolle_set', to='medlem.Lokallag')),
                ('medlem', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='medlem.Medlem')),
            ],
            options={
                'ordering': ['rolletype', 'medlem'],
                'verbose_name_plural': 'roller',
            },
        ),
        migrations.CreateModel(
            name='Rolletype',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('namn', models.CharField(max_length=64, verbose_name='rollenamn')),
            ],
            options={
                'ordering': ['namn'],
                'verbose_name_plural': 'rolletyper',
            },
        ),
        migrations.CreateModel(
            name='Val',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('tittel', models.CharField(max_length=100, unique=True, verbose_name='kort forklaring')),
                ('forklaring', models.TextField(blank=True, verbose_name='lang forklaring')),
            ],
            options={
                'verbose_name_plural': 'val',
            },
        ),
        migrations.AddField(
            model_name='rolle',
            name='rolletype',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='medlem.Rolletype'),
        ),
        migrations.AddField(
            model_name='medlem',
            name='lokallagsrolle',
            field=models.ManyToManyField(blank=True, related_name='rollemedlem', through='medlem.Rolle', to='medlem.Lokallag'),
        ),
        migrations.AddField(
            model_name='medlem',
            name='user',
            field=models.OneToOneField(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL, verbose_name='innloggingsbrukar'),
        ),
        migrations.AddField(
            model_name='medlem',
            name='val',
            field=models.ManyToManyField(blank=True, to='medlem.Val'),
        ),
        migrations.AddField(
            model_name='medlem',
            name='verva_av',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='har_verva', to='medlem.Medlem'),
        ),
        migrations.AddField(
            model_name='lokallagovervaking',
            name='medlem',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='medlem.Medlem'),
        ),
        migrations.AddField(
            model_name='giro',
            name='medlem',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='giroar', to='medlem.Medlem'),
        ),
    ]



from django.conf.urls import url, include
from rest_framework.authtoken.views import obtain_auth_token
from rest_framework.routers import DefaultRouter

from medlemssys.api import views


router = DefaultRouter()
router.register(r'medlem', views.MedlemViewSet)

urlpatterns = [
    url(r'^', include(router.urls)),
    url(r'innmelding', views.medlem_innmelding, name='api-innmelding'),
    url(r'token', obtain_auth_token),
]



# -*- coding: utf-8 -*-
# vim: ts=4 sts=4 expandtab ai

# Copyright 2009-2016 Odin Hørthe Omdal

# This file is part of Medlemssys.

# Medlemssys is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# Medlemssys is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with Medlemssys.  If not, see <http://www.gnu.org/licenses/>.

from __future__ import unicode_literals

import random
import datetime

from django.contrib.auth.models import User
from django.core.urlresolvers import reverse
from django.db import models, transaction
from django.forms import ModelForm
from django.template.defaultfilters import slugify
from django.utils import timezone
from django.utils.translation import ugettext_lazy as _

from medlemssys.behaviour import get_behaviour
from medlemssys.innhenting import mod10


def generate_password(length=7):
    return ''.join(random.choice('abcdefghjkmnpqrstuvwxyz23456789') for x in range(length))


class Val(models.Model):
    tittel = models.CharField(_("kort forklaring"), max_length=100, unique=True)
    forklaring = models.TextField(_("lang forklaring"), blank=True)

    @property
    def namn(self):
        return self.tittel

    class Meta:
        verbose_name_plural = "val"

    def __unicode__(self):
        return self.tittel

    def num_medlem(self):
        return self.medlem_set.count()
    num_medlem.short_description = "medlemstal"


class Lokallag(models.Model):
    namn = models.CharField(_("namn"), max_length=255, unique=True)
    slug = models.SlugField(_("nettnamn"), max_length=255, unique=True, blank=True)
    kommunes = models.CharField(_("kommunar"), max_length=255, blank=True,
                   help_text="""Kommaseparert med stor forbokstav, t.d. "Stavanger, Sandnes".""")
    fylke = models.CharField(_("fylke"), max_length=255, blank=True,
                   help_text="""Berre eitt fylke, stor forbokstav. Overstyrer kommuner visst denne finst.""")
    andsvar = models.CharField(_("andsvar"), max_length=255, blank=True)
    lokalsats = models.IntegerField(_("lokalsats"), default=0)

    aktivt = models.BooleanField(_("aktivt"), default=True)
    epost = models.EmailField(_("epost"), blank=True,
        help_text=_("""Eiga epostliste til styret. Tek prioritet
        over å senda enkeltvis til dei som er registrert som styre."""))

    class Meta:
        verbose_name_plural = "lokallag"
        ordering = ['-aktivt', 'namn']

    def num_medlem(self):
        return self.medlem_set.betalande().count()
    num_medlem.short_description = "medlemstal"

    def listing(self):
        return reverse('lokallag_home', args=(self.slug,))

    def styreepostar(self):
        if self.epost:
            return [self.epost]
        return self.rollemedlem.exclude(epost='').values_list('epost',
                                                              flat=True)

    def styret(self):
        return self.rolle_set.all()

    def styret_admin(self):
        styret = ["{} ({})".format(x.medlem.admin_change(), x.rolletype)
                  for x in self.styret()]
        return ', '.join(styret)
    styret_admin.short_description = _("Styret")
    styret_admin.allow_tags = True

    def save(self, *args, **kwargs):
        if not self.slug or self.slug == "":
            self.slug = slugify(self.namn)
        super(Lokallag, self).save(*args, **kwargs)

    def get_absolute_url(self):
        return self.listing()

    def __unicode__(self):
        return self.namn


class Innmeldingstype(models.Model):
    namn = models.CharField(_("namn"), max_length=255, unique=True)

    class Meta:
        verbose_name_plural = "innmeldingstypar"

    def __unicode__(self):
        return self.namn


STATUSAR = (
    ("M", "Vanleg medlem"),
    ("I", "Infoperson"),
    ("L", "Livstidsmedlem"),
)

INNMELDINGSTYPAR = (
    ("H", "Heimesida"),
    ("M", "Målferd"),
    ("V", "Vervekampanje"),
    ("F", "Flygeblad"),
    ("L", "Lagsskiping/årsmøte"),
    ("D", "Direkteverva"),
    ("S", "SMS"),
    ("O", "Vitjing"),
    ("A", "Anna"),
    ("U", "--"),
)


class MedlemManager(models.Manager):
    def get_queryset(self):
        return get_behaviour().queryset_class(self.model)

    def __getattr__(self, attr, *args):
        if attr.startswith("_"):
            raise AttributeError
        return getattr(self.get_queryset(), attr, *args)


class Medlem(models.Model):
    fornamn = models.CharField(_("fornamn"), max_length=255)
    mellomnamn = models.CharField(_("mellomnamn"), max_length=255,
            blank=True)
    etternamn = models.CharField(_("etternamn"), max_length=255)
    fodt = models.IntegerField(_("født"), blank=True, null=True)
    # Hadde denne før, men importering gjorde folk so gamle då: default=date.today().year - 17,

    # Kontakt
    postnr = models.CharField(
        _("postnr"), max_length=4,
        help_text="'0000' viss norsk adresse manglar")
    epost = models.CharField(_("epost"), max_length=255,
            blank=True)
    postadr = models.CharField(
        _("postadresse"), max_length=255, blank=True, null=True,
        help_text="Skriv utanlands-/skuleadresse i borte-addresse")
    ekstraadr = models.CharField(_("ekstraadresse"), max_length=255,
            blank=True)
    mobnr = models.CharField(_("mobiltelefon"), max_length=50,
            blank=True)
    heimenr = models.CharField(_("heimetelefon"), max_length=50,
            blank=True)

    bortepostnr = models.CharField(
            _("borte-postnr"), max_length=255, blank=True, null=True,
            help_text="T.d. '0864' eller 'c/o John Smith, 50012 OX-5, Oxford England'")
    borteadr = models.CharField(
            _("borte-adresse"), max_length=255, blank=True, null=True,
            help_text="Dersom borte-adressene er sett, vert post sendt her. For t.d. hybel- og utenlandsadresser.")

    # Om medlemen
    gjer = models.CharField(_("gjer"), max_length=255,
        blank=True)
    kjon = models.CharField(_("kjønn"), max_length=1, choices=(("M", "Mann"),
        ("K", "Kvinne"), ("U", "Udefinert")), default="U")
    merknad = models.TextField(_("merknad"), blank=True, default="")

    # Spesialfelt, denormalisert felt frå Giro
    _siste_medlemspengar = models.PositiveIntegerField(
        blank=True, null=True, editable=False, default=None)

    # Medlemsskapet
    innmeldt_dato = models.DateField(
        _("innmeldt"), default=datetime.date.today)
    utmeldt_dato = models.DateField(
        _("utmeldt"), blank=True, null=True, default=None)
    status = models.CharField(_("medlstatus"), max_length=1,
            choices=STATUSAR, default="M")
    innmeldingstype = models.CharField(_("innmtype"), max_length=1,
            choices=INNMELDINGSTYPAR, default='U')
    innmeldingsdetalj = models.CharField(_("detalj om innmelding"), max_length=255,
        blank=True, help_text=_("Skriv inn vervemedlem i hakeparantesar ([1234])"))
    verva_av = models.ForeignKey('Medlem', related_name='har_verva', blank=True, null=True)
    betalt_av = models.ForeignKey('Medlem', related_name='betalar_for', blank=True, null=True)

    # Tilkopla felt
    lokallag = models.ForeignKey(Lokallag,
        blank=True, null=True, on_delete=models.SET_NULL)
    val = models.ManyToManyField(Val, blank=True)
    lokallagsrolle = models.ManyToManyField(Lokallag,
        through='Rolle', related_name='rollemedlem', blank=True)

    user = models.OneToOneField(User, verbose_name=_("innloggingsbrukar"),
                                blank=True, null=True)
    oppretta = models.DateTimeField(_("oppretta"), auto_now_add=True)
    oppdatert = models.DateTimeField(_("oppdatert"), auto_now=True)
    nykel = models.CharField(_("nykel"), max_length=255,
            blank=True, default=generate_password)

    @property
    def stad(self):
        try:
            return PostNummer.objects.get(postnr=self.postnr).poststad
        except PostNummer.DoesNotExist:
            return "?"

    @property
    def bortestad(self):
        try:
            return PostNummer.objects.get(postnr=self.bortepostnr).poststad
        except PostNummer.DoesNotExist:
            return "?"

    @property
    def bukommune(self):
        postnr = self.bortepostnr or self.postnr
        try:
            return PostNummer.objects.get(postnr=postnr).kommune
        except PostNummer.DoesNotExist:
            return "?"

    @property
    def bufylke(self):
        postnr = self.bortepostnr or self.postnr
        try:
            return PostNummer.objects.get(postnr=postnr).fylke
        except PostNummer.DoesNotExist:
            return "?"

    # Managers
    objects = MedlemManager()

    class Meta:
        verbose_name_plural = "medlem"
        ordering = ['-id']

    def __unicode__(self):
        return ' '.join(
            [ x for x in [self.fornamn,
                          self.mellomnamn,
                          self.etternamn] if x ])
    __unicode__.admin_order_field = 'etternamn'

    def get_absolute_url(self):
        return reverse('medlem_edit', args=(self.pk, self.nykel))

    def er_innmeldt(self):
        if (self.utmeldt_dato):
            return self.utmeldt_dato > datetime.date.today()
        return True
    er_innmeldt.short_description = _("Innmeld")
    er_innmeldt.boolean = True

    def er_teljande(self):
        return self.fodt > (datetime.date.today().year - 26)
    er_teljande.short_description = _("Teljande")
    er_teljande.boolean = True

    def er_gamal(self):
        return self.fodt < (datetime.date.today().year - 23)
    er_teljande.short_description = _("Gamal")
    er_teljande.boolean = True

    def alder(self):
        if not self.fodt:
            return None
        alder = datetime.date.today().year - self.fodt
        if alder >= 0 and alder < 120:
            return alder
        return None

    def pronomen(self):
        if self.kjon == 'M':
            return "han"
        elif self.kjon == 'K':
            return "ho"
        return "ho/han"

    def fodt_farga(self):
        if self.alder() == None:
            return "?"
        if not self.er_teljande():
            return "<span class='ikkje-teljande' title='Medlemen er ikkje " + \
               "teljande (%d år)'>%s</span>" % (self.alder(), self.fodt)
        elif self.er_gamal():
            return "<span class='er-gamal' title='Medlemen er gamal, men " + \
        "framleis teljande (%d år)'>%s</span>" % (self.alder(), self.fodt)
        return "<span title='%d år'>%d</span>" % (self.alder(), self.fodt)
    fodt_farga.short_description = _("Født")
    fodt_farga.admin_order_field = 'fodt'
    fodt_farga.allow_tags = True

    def har_betalt(self):
        if (self.giroar.filter(gjeldande_aar=datetime.date.today().year,
                                 innbetalt__isnull=False)).exists():
            return True
        else:
            return False
    har_betalt.short_description = _("Betalt")
    har_betalt.boolean = True

    def status_html(self):
        return "<abbr title='%s'>%s</abbr>" % (self.get_status_display(), self.status)
    status_html.short_description = _("Status")
    status_html.admin_order_field = 'status'
    status_html.allow_tags = True

    def full_adresse(self, **kwargs):
        if 'namn' not in kwargs:
            kwargs['namn'] = False
        return self.full_postadresse(heimeadresse=True, **kwargs)

    def full_postadresse(self, namn=True, heimeadresse=False, as_list=False):
        adr = []
        if namn:
            adr.append(unicode(self))
        postnr, adresse, poststad, ekstra = self.postnr, self.postadr, self.stad, self.ekstraadr
        if (self.borteadr or self.bortepostnr) and not heimeadresse:
            postnr, adresse, poststad, ekstra = self.bortepostnr, self.borteadr, self.bortestad, ''
        if adresse:
            adr.append(adresse)
        if "0005" < postnr < "9995":
            if ekstra:
                adr.append(ekstra)
            adr.append(postnr + " " + poststad)
        else:
            # Jækelen er i utlandet
            if ekstra:
                adr.append(ekstra)
            if len(postnr) > 4:
                adr.append(postnr)
        if as_list:
            return adr
        return "\n".join(adr)

    def full_betalingsadresse(self, **kwargs):
        if self.betalt_av:
            return self.betalt_av.full_betalingsadresse(**kwargs)
        return self.full_postadresse(**kwargs)

    def set_val(self, tittel, add=True):
        if add:
            self.val.add(Val.objects.get_or_create(tittel=tittel)[0])
        else:
            self.val.remove(Val.objects.get(tittel=tittel))

    def val_exists(self, tittel):
        return self.val.filter(tittel=tittel).exists()

    def lokallag_display(self):
        if not self.lokallag_id:
            return "(ingen)"
        return unicode(self.lokallag)

    def proposed_lokallag(self):
        """
        Lokallag that the system would choose itself if it had
        to go only on the postnr (or bortepostnr) of the member.
        """
        alder = self.alder()
        lags = set(Lokallag.objects.filter(
            models.Q(fylke=self.bufylke)
            | models.Q(kommunes__icontains=self.bukommune)))
        ordered = []
        if self.lokallag in lags:
            lags.remove(self.lokallag)
            ordered = [self.lokallag]
        last = None
        for lag in lags:
            if 'student' in lag.namn.lower():
                if alder > 18:
                    ordered.insert(0, lag)
                else:
                    last = lag
                continue
            ordered.append(lag)
        if last:
            ordered.append(last)
        return ordered

    def gjeldande_giro(self, year=datetime.date.today().year):
        try:
            return self.giroar.filter(gjeldande_aar=year)[0]
        except IndexError:
            return None

    def admin_url(self):
        return reverse('admin:medlem_medlem_change', args=(self.pk,))

    def admin_change(self):
        return '<a href="{0}">{1}</a>'.format(self.admin_url(), self)
    admin_change.short_description = _("Medlem")
    admin_change.admin_order_field = 'medlem'
    admin_change.allow_tags = True

    def save(self, *args, **kwargs):
        if self.mobnr and ' ' in self.mobnr:
            self.mobnr = ''.join(self.mobnr.split())
        super(Medlem, self).save(*args, **kwargs)


@transaction.atomic
def update_denormalized_fields():
    for date in (Giro.objects.values('gjeldande_aar')
                 .distinct().order_by('gjeldande_aar')):
        (Medlem.objects.filter(
              giroar__gjeldande_aar=date['gjeldande_aar'],
              giroar__innbetalt__isnull=False)
           .update(_siste_medlemspengar=date['gjeldande_aar']))

    (Medlem.objects.filter(giroar__isnull=True)
       .update(_siste_medlemspengar=None))
    (Giro.objects.filter(
          innbetalt__isnull=True,
          medlem__utmeldt_dato__gte=datetime.date.today())
       .delete())


class EndraMedlemForm(ModelForm):
    class Meta:
        model = Medlem
        fields = ('fornamn', 'mellomnamn', 'etternamn',
                  'fodt', 'postadr', 'ekstraadr', 'postnr', 'epost', 'mobnr',
                  'borteadr', 'bortepostnr')


class InnmeldingMedlemForm(ModelForm):
    class Meta:
        model = Medlem
        fields = ('fornamn', 'etternamn', 'postnr', 'epost', 'mobnr',)

    def __init__(self, *args, **kwargs):
        super(InnmeldingMedlemForm, self).__init__(*args, **kwargs)
        #self.fields["lokallag"].initial = MyModel.objects.get(id=1)

    def save(self, commit=True, *args, **kwargs):
        m = super(InnmeldingMedlemForm, self).save(commit=False)
        m.lokallag = Lokallag.objects.get(pk=1)
        if commit:
            m.save()
        return m



KONTI = (
    ('A', "Medlemskonto (KID)"),
    ('M', "Medlemskonto"),
    ('K', "Kassa"),
    ('B', "Brukskonto"),
    ('S', "SMS"),
)
HENSIKTER = (
    ('P', "Medlemspengar"),
    ('G', "Gåve"),
    ('T', "Tilskiping"),
    ('A', "Anna"),
)
GIRO_STATUSAR = (
    ('V', "Ventar"),
    ('1', "Epost sendt"),
    ('E', "(E) Sendingsfeil"),
    ('M', "Manuelt postlagt"),
    ('F', "Ferdig"),
    ('U', "Utgått ubetalt"),
)


def this_year():
    return datetime.date.today().year

class Giro(models.Model):
    medlem = models.ForeignKey(Medlem, related_name='giroar')
    belop = models.PositiveIntegerField(_("Beløp"))
    innbetalt_belop = models.PositiveIntegerField(_("Innbetalt beløp"), default=0)

    # XXX This is not really unique, but unique per oppretta_year and oppretta_year - 1
    kid = models.CharField(_("KID-nummer"), max_length=255, blank=True, unique=True)

    oppretta = models.DateTimeField(_("oppretta"), auto_now_add=True)
    oppdatert = models.DateTimeField(_("oppatert"), auto_now=True)
    innbetalt = models.DateField(_("Dato betalt"), blank=True, null=True)
    konto = models.CharField(_("Konto"), max_length=1, choices=KONTI, default="M")
    hensikt = models.CharField(_("Hensikt"), max_length=1, choices=HENSIKTER, default="P")
    desc = models.TextField(_("Forklaring"), blank=True, default="")

    status = models.CharField(_("Status"), max_length=1, choices=GIRO_STATUSAR, default="V")
    gjeldande_aar = models.PositiveIntegerField(
        _("Gjeldande år"), default=this_year)

    class Meta:
        verbose_name_plural = "giroar"
        ordering = ('-oppdatert', '-innbetalt', '-pk')

    def __unicode__(self):
        betalt = "IKKJE BETALT"
        if self.betalt():
            betalt = "betalt"
        return "%s, %s (%s)" % (self.medlem, self.gjeldande_aar, betalt)

    def betalt(self):
        if self.innbetalt and self.innbetalt_belop >= self.belop:
            return True
        return False

    def admin_change(self):
        url = reverse('admin:medlem_giro_change', args=(self.pk,))
        belop = str(self.innbetalt_belop)
        if self.belop != self.innbetalt_belop:
            belop = '{0}/{1}'.format(self.innbetalt_belop, self.belop)
        if self.betalt():
            return '<a href="{url}">{img} {belop}</a>'.format(
                url=url,
                img='<img src="/static/admin/img/icon-yes.svg" alt="Betalt">',
                belop=belop)
        return '<a href="{url}">{img} {belop}</a>'.format(
            url=url,
            img='<img src="/static/admin/img/icon-no.svg" alt="Ikkje betalt">',
            belop=belop)
    admin_change.short_description = _("Giro")
    admin_change.admin_order_field = 'giro'
    admin_change.allow_tags = True

    def save(self, *args, **kwargs):
        if not self.medlem.er_innmeldt() and self.innbetalt is None:
            if self.pk:
                self.delete()
            return

        if self.betalt():
            self.status = 'F'

        if len(self.kid) < 1:
            super(Giro, self).save(*args, **kwargs)
            self.kid = str(int(self.pk) % 100000).zfill(5)
            self.kid = mod10.kid_add_controlbit(self.kid)

        # Fiks det denormaliserte feltet til medlemen, som fortel um siste
        # medlemspengebetaling.
        if self.hensikt == 'P' \
                and self.innbetalt is not None \
                and (self.medlem._siste_medlemspengar is None \
                    or self.medlem._siste_medlemspengar < self.gjeldande_aar):
            self.medlem._siste_medlemspengar = self.gjeldande_aar
            self.medlem.save()

        super(Giro, self).save(*args, **kwargs)


class Rolletype(models.Model):
    namn = models.CharField(_("rollenamn"), max_length=64)

    def __unicode__(self):
        return self.namn

    class Meta:
        verbose_name_plural = "rolletyper"
        ordering = ['namn']

    def rolle_num(self):
        return self.rolle_set.count()
    rolle_num.short_description = "Folk med rolle"


class Rolle(models.Model):
    medlem = models.ForeignKey(Medlem)
    lokallag = models.ForeignKey(Lokallag, related_name='rolle_set')
    rolletype = models.ForeignKey(Rolletype, blank=True, null=True)

    class Meta:
        verbose_name_plural = "roller"
        ordering = ['rolletype', 'medlem']

    def __unicode__(self):
        return "%s, %s (%s)" % (self.medlem, self.lokallag, self.rolletype)


class LokallagOvervaking(models.Model):
    medlem = models.ForeignKey(Medlem, blank=True, null=True)
    epost = models.CharField(_("epost"), max_length=255, blank=True,
            help_text="""Vert brukt dersom medlem ikkje er satt""")
    lokallag = models.ForeignKey(
        Lokallag, related_name='lokallag_overvaking_set')
    deaktivert = models.DateTimeField(blank=True, null=True)
    sist_oppdatert = models.DateTimeField(auto_now=True)

    class Meta:
        verbose_name_plural = "lokallagovervakingar"

    def epostar(self):
        if self.medlem:
            return [self.medlem.epost]
        elif self.epost:
            return [self.epost]
        else:
            return self.lokallag.styreepostar()

    def epostar_admin(self):
        return ', '.join(self.epostar())
    epostar_admin.short_description = "Brukt epost"


    def __unicode__(self):
        ekstra = ""
        if self.deaktivert and self.deaktivert < timezone.now():
            ekstra += " (deaktivert)"
        if not len(self.epostar()):
            ekstra += " (tom)"

        if self.medlem:
            return "%s overvakar %s%s" % (self.medlem, self.lokallag, ekstra)
        elif self.epost:
            return "%s overvakar %s%s" % (self.epost, self.lokallag, ekstra)
        elif self.lokallag.styreepostar():
            return "Styret overvakar %s%s" % (self.lokallag, ekstra)
        else:
            return "Overvaking %s%s" % (self.lokallag, ekstra)

class PostNummer(models.Model):
    postnr = models.CharField(max_length=6)
    poststad = models.CharField(max_length=50)
    bruksomrade = models.CharField(max_length=50)
    folketal = models.SmallIntegerField(null=True, blank=True)
    bydel = models.CharField(max_length=50)
    kommnr = models.CharField(max_length=50)
    kommune = models.CharField(max_length=50)
    fylke = models.CharField(max_length=50)
    lat = models.FloatField()
    lon = models.FloatField()
    datakvalitet = models.SmallIntegerField()
    sist_oppdatert = models.DateTimeField(null=True, blank=True)

    class Meta:
        verbose_name_plural = "postnummer"

    def __unicode__(self):
        return "{0} {1}".format(self.postnr, self.poststad)

    def num_medlem(self):
        return Medlem.objects.filter(postnr=self.postnr).count()
    num_medlem.short_description = "medlemstal"

    def num_teljande_medlem(self):
        return Medlem.objects.filter(postnr=self.postnr).teljande().count()
    num_teljande_medlem.short_description = "teljande"



# -*- coding: utf-8 -*-
# vim: ts=4 sw=4 sts=4 expandtab ai

# Copyright 2009-2014 Odin Hørthe Omdal

# This file is part of Medlemssys.

# Medlemssys is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# Medlemssys is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with Medlemssys.  If not, see <http://www.gnu.org/licenses/>.
import csv
import datetime
import os
from cStringIO import StringIO

from reversion import revisions as reversion

from django.conf import settings
from django.contrib.admin import helpers
from django.db import transaction
from django.forms.models import model_to_dict
from django.http import HttpResponse
from django.template import Context, Template
from django.template.response import TemplateResponse
from django.utils import timezone
from django.utils.translation import ugettext as _
from reportlab.lib.units import cm #, mm
from reportlab.pdfbase import pdfmetrics
from reportlab.pdfbase.ttfonts import TTFont
from reportlab.pdfgen import canvas
from reportlab.platypus import Paragraph
from reportlab.lib.styles import getSampleStyleSheet

from medlemssys.innhenting import mod10
from medlemssys.medlem.models import Giro

import admin # reversion
from .forms import SuggestedLokallagForm


@transaction.atomic
def members_meld_ut(modeladmin, request, queryset):
    today = timezone.now().date()
    with reversion.create_revision():
        reversion.set_comment("Meld ut medlemar")
        reversion.set_user(request.user)
        for m in queryset:
            if not m.utmeldt_dato:
                m.utmeldt_dato = today
            if not m.fornamn.lower().startswith('utmeld'):
                m.fornamn = u'UTMELD ' + m.fornamn
            m.save()
members_meld_ut.short_description = "Meld ut"


def simple_member_list(modeladmin, request, queryset):
    response = HttpResponse(content_type='text/csv; charset=utf-8')
    response['Content-Disposition'] = 'filename=medlemer.csv'

    dc = csv.writer(response, quoting=csv.QUOTE_ALL)
    dc.writerow(["Namn",
                 u"Fødd".encode('utf-8'),
                 "Heimeadresse",
                 "Postadresse",
                 "Poststad",
                 "Mobiltelefon",
                 "Epost",
                 "Lokallag",
                 "Medlemsnummer",
                 "Betalt",
                 u"Beløp".encode('utf-8'),
                 "KID"])
    for m in queryset:
        giro = m.gjeldande_giro()
        belop = '-'
        innbetalt = '-'
        kid = ''
        if giro:
            belop = giro.belop
            innbetalt = giro.innbetalt_belop
            kid = giro.kid
        postadr, poststad = '', ''
        adr = m.full_postadresse(namn=False, as_list=True)
        if len(adr) == 1:
            poststad = adr[0]
        elif len(adr) > 1:
            postadr = ', '.join(adr[:-1])
            poststad = adr[-1]
        a = [m,
             m.fodt,
             m.full_adresse(namn=False),
             postadr,
             poststad,
             m.mobnr,
             m.epost,
             m.lokallag_display(),
             m.id,
             innbetalt,
             belop,
             kid]
        dc.writerow([unicode(s).encode('utf-8') for s in a])
    return response
simple_member_list.short_description = "Enkel medlemsliste"


def csv_list(modeladmin, request, queryset):
    response = HttpResponse(content_type='text/csv; charset=utf-8')
    response['Content-Disposition'] = 'filename=dataliste.csv'

    dc = csv.writer(response)
    dc.writerow(model_to_dict(queryset[0]).keys())
    for m in queryset:
        a = model_to_dict(m).values()
        dc.writerow([unicode(s).encode('utf-8') for s in a])

    return response
csv_list.short_description = "Full dataliste"


def giro_list(modeladmin, request, queryset):
    response = HttpResponse(content_type='text/csv; charset=utf-8')
    response['Content-Disposition'] = 'filename=giroar.csv'

    dc = csv.writer(response, quoting=csv.QUOTE_ALL)
    dc.writerow(["Namn",
                 "Heimeadresse",
                 "Postadresse",
                 "Poststad",
                 "Betalt",
                 "Dato",
                 "Lokallag",
                 u"Fødd".encode('utf-8'),
                 "Epost",
                 "Mobnr",
                 u"For år".encode('utf-8'),
                 "Korleis",
                 "KID",
                 u"Beløp".encode('utf-8'),
                 "Medlem-ID",
                 "Giro-ID",
                 ])
    for g in queryset:
        postadr, poststad = '', ''
        adr = g.medlem.full_postadresse(namn=False, as_list=True)
        if len(adr) == 1:
            poststad = adr[0]
        elif len(adr) > 1:
            postadr = ', '.join(adr[:-1])
            poststad = adr[-1]

        a = [g.medlem,
             g.medlem.full_adresse(namn=False),
             postadr,
             poststad,
             g.innbetalt_belop,
             g.innbetalt,
             g.medlem.lokallag_display(),
             g.medlem.fodt,
             g.medlem.epost,
             g.medlem.mobnr,
             g.gjeldande_aar,
             g.konto,
             g.kid,
             g.belop,
             g.medlem.id,
             g.pk]

        dc.writerow([unicode(s).encode('utf-8') for s in a])

    return response
giro_list.short_description = "Enkel giroliste"


@transaction.atomic
def pdf_giro(modeladmin, request, queryset):
    # User has already written some text, make PDF
    if request.POST.get('post'):
        pdfmetrics.registerFont(TTFont('OCRB', os.path.dirname(__file__) + '/../giro/OCRB.ttf'))
        response = HttpResponse(content_type='application/pdf')
        response['Content-Disposition'] = 'filename=girosending.pdf'

        buf = StringIO()

        # Create the PDF object, using the StringIO object as its "file."
        pdf = canvas.Canvas(buf)

        if not request.POST.get('ink-utmeld'):
            queryset = queryset.filter(medlem__utmeldt_dato__isnull=True)
        if not request.POST.get('ink-betalt'):
            queryset = queryset.exclude(innbetalt__isnull=False)

        for giro in queryset:
            m = giro.medlem
            if not giro:
                continue
            if request.POST.get('pdf_type') == 'medlemskort':
                pdf = _giro(pdf, request, m, giro)
                pdf = _medlemskort(pdf, request, m, giro)
            elif request.POST.get('pdf_type') == 'giro':
                pdf = _giro(pdf, request, m, giro)
            else:
                pdf = _giro_faktura(pdf, request, m, giro)
            pdf.showPage()
            if request.POST.get('merk-postlagt'):
                giro.status = 'M'
                giro.save()

        # Close the PDF object cleanly.
        pdf.save()

        # Get the value of the StringIO buffer and write it to the response.
        pdf = buf.getvalue()
        buf.close()
        response.write(pdf)
        return response

    opts = modeladmin.model._meta
    app_label = opts.app_label
    n_utmelde = queryset.filter(medlem__utmeldt_dato__isnull=False).count()
    n_betalte = queryset.filter(innbetalt__isnull=False).count()
    g_frist = datetime.date.today() + datetime.timedelta(30)

    title = _("Lag giro-PDF-ar ({c} giroar)".format(c=queryset.count()))

    context = {
        "title": title,
        "queryset": queryset,
        "opts": opts,
        "app_label": app_label,
        "action_checkbox_name": helpers.ACTION_CHECKBOX_NAME,
        "n_utmelde": n_utmelde,
        "n_betalte": n_betalte,
        "g_frist": g_frist,
    }

    return TemplateResponse(request, 'admin/pdf_giro.html', context,
            current_app=modeladmin.admin_site.name)
pdf_giro.short_description = "Lag giro-PDF"


def _pdf_p(pdf, text, x, y, size_w=None, size_h=None):
    if not size_w:
        size_w = 19
    if not size_h:
        size_h = size_w

    style = getSampleStyleSheet()['BodyText']
    p = Paragraph(text, style)
    used_w, used_h = p.wrap(size_w*cm, size_h*cm)
    p.wrapOn(pdf, size_w*cm, size_h*cm)
    p.drawOn(pdf, x*cm, y*cm - used_h)


def _giro_faktura(pdf, request, m, giro):
    pdf.setFont('Helvetica', 16)
    pdf.drawString(1.0*cm, 16*cm, u"%s" % request.POST.get('title'))

    pdf.setFontSize(11)
    text = request.POST.get('text')
    text_content = Template(text).render(Context({'medlem': m, 'giro': giro})).replace('\n', '<br/>')
    _pdf_p(pdf, text_content, 1, 15.5, size_w=19, size_h=13)
    _pdf_p(pdf, m.full_betalingsadresse().replace('\n', '<br/>\n'), 1, 26, size_w=8, size_h=6)
    _pdf_p(pdf, u"""\
        Kundenr: {m.pk}<br/>
        Fakturanr: {g.pk}<br/>
        Fakturadato: {now}<br/>
        Organisasjonsnr: {orgnr}<br/>
        Betalingsfrist: {frist}<br/>
        <br/>
        Til konto: <b>{kontonr}</b><br/>
        KID-nummer: <b>{g.kid}</b><br/>
        Å betala: <b>{g.belop},00</b><br/>
        """.format(m=m,
                   g=giro,
                   kontonr=settings.KONTONUMMER,
                   orgnr=settings.ORGNUMMER,
                   now=datetime.date.today(),
                   frist=request.POST.get('frist')),
        15, 26, size_w=4, size_h=6)
    return pdf


def _medlemskort(pdf, request, m, giro):
    # Medlemskort
    pdf.setFont('Helvetica', 12)
    pdf.drawString(13.0*cm, 14.9*cm, u"%s" % unicode(m))

    pdf.setFont('OCRB', 11)

    pdf.drawString(13.8*cm, 14.3*cm, u"%s" % m.pk)
    pdf.drawString(18.1*cm, 14.3*cm, u"%s" % giro.gjeldande_aar)
    pdf.drawString(13.0*cm, 12.7*cm, u"%s" % giro.belop)
    pdf.drawString(14.8*cm, 12.7*cm, u"%s" % '00')
    return pdf


def _giro(pdf, request, m, giro):
    pdf.setFont('Helvetica', 16)
    pdf.drawString(1.0*cm, 26*cm, u"%s" % request.POST.get('title'))
    pdf.setFontSize(12)

    text_template = Template(request.POST.get('text'))
    text_context = Context({'medlem': m, 'giro': giro})
    text_content = text_template.render(text_context).replace('\n', '<br/>')

    _pdf_p(pdf, text_content, 1, 25.5, size_w=18, size_h=13)

    pdf.setFont('OCRB', 11)
    # Giro
    tekst = pdf.beginText(1.2*cm, 5.5*cm)
    for adrdel in m.full_betalingsadresse().split('\n'):
        tekst.textLine(adrdel.strip())
    pdf.drawText(tekst)
    if m.har_betalt():
        pdf.drawString(18*cm, 12.8*cm, "BETALT")
        pdf.setFillColorRGB(0, 0, 0)
        pdf.rect(0,  5.3*cm, 26*cm, 0.2*cm, stroke=False, fill=True)
        pdf.rect(0, 4.85*cm, 26*cm, 0.2*cm, stroke=False, fill=True)
        pdf.rect(0,  1.9*cm, 26*cm, 0.2*cm, stroke=False, fill=True)
        pdf.rect(0, 1.45*cm, 26*cm, 0.2*cm, stroke=False, fill=True)
    else:
        pdf.drawString(17.1*cm, 9.3*cm, u"%s" % request.POST.get('frist'))
        pdf.drawString(5.0*cm,  1.58*cm, u"%s" % giro.kid)
        pdf.drawString(8.5*cm,  1.58*cm, u"%s" % giro.belop)
        pdf.drawString(10.6*cm, 1.58*cm, u"%s" % '00')
        pdf.drawString(11.9*cm, 1.58*cm,
                       u"%s" % mod10.mod10(unicode(giro.belop) + '00'))
        pdf.drawString(13.2*cm, 1.58*cm, u"%s" % settings.KONTONUMMER)

    return pdf


@transaction.atomic
def lag_giroar(modeladmin, request, queryset):
    year = datetime.date.today().year
    if not request.POST.get('post'):
        opts = modeladmin.model._meta
        app_label = opts.app_label
        n_utmelde = queryset.filter(utmeldt_dato__isnull=False).count()
        n_allereie_giro = queryset.filter(giroar__gjeldande_aar=year).count()

        title = _("Lag giroar")

        context = {
            "title": title,
            "queryset": queryset,
            "opts": opts,
            "app_label": app_label,
            "action_checkbox_name": helpers.ACTION_CHECKBOX_NAME,
            "n_utmelde": n_utmelde,
            "n_allereie_giro": n_allereie_giro,
        }

        return TemplateResponse(request, "admin/lag_giroar.html", context,
                current_app=modeladmin.admin_site.name)

    if not request.POST.get("ink-utmeld"):
        queryset = queryset.filter(utmeldt_dato__isnull=True)

    queryset = queryset.exclude(giroar__gjeldande_aar=year)

    with reversion.create_revision():
        reversion.set_comment("Opprett giroar admin action")
        reversion.set_user(request.user)
        for m in queryset:
            g = Giro(belop=request.POST.get('belop'), medlem=m)
            g.save()
lag_giroar.short_description = "Opprett giroar"


def giro_status_ferdig(modeladmin, request, queryset):
    with reversion.create_revision():
        reversion.set_comment("Giro status ferdig admin action")
        reversion.set_user(request.user)
        today = datetime.date.today()
        for g in queryset:
            g.status='F'
            if not g.innbetalt_belop:
                g.innbetalt_belop = g.belop
            if not g.innbetalt:
                g.innbetalt = today
            g.save()
giro_status_ferdig.short_description = "Sett giro betalt"


def giro_status_postlagt(modeladmin, request, queryset):
    with reversion.create_revision():
        for g in queryset:
            g.status='M'
            g.save()
        reversion.set_comment("Giro status postlagt admin action")
        reversion.set_user(request.user)
giro_status_postlagt.short_description = "Sett girostatus 'Manuelt postlagt'"


def giro_status_ventar(modeladmin, request, queryset):
    with reversion.create_revision():
        for g in queryset:
            g.status='V'
            g.save()
        reversion.set_comment("Giro status ventar admin action")
        reversion.set_user(request.user)
giro_status_ventar.short_description = "Sett girostatus 'Ventar'"


@transaction.atomic
def suggest_lokallag(modeladmin, request, queryset):
    if request.POST.get('post'):
        updated = []
        with reversion.create_revision():
            reversion.set_comment("Endra lokallag med suggest_lokallag")
            reversion.set_user(request.user)
            for m in queryset:
                pk = "lokallag_{0}".format(m.pk)
                if pk not in request.POST:
                    continue
                try:
                    new_lokallag_id = int(request.POST[pk])
                except ValueError:
                    continue
                if m.lokallag_id == new_lokallag_id:
                    continue
                m.lokallag_id = new_lokallag_id
                m.save()
                updated.append(m)
        modeladmin.message_user(
            request,
            "{} brukarar med oppdaterte lokallag".format(len(updated)))
        return None
    already_ok = []
    suggested_new = []
    totally_new = []
    nothing = []
    for m in queryset:
        m.prop = m.proposed_lokallag()
        if m.prop and m.lokallag == m.prop[0]:
            already_ok.append(m)
        elif m.lokallag in m.prop:
            suggested_new.append(m)
        elif m.prop:
            totally_new.append(m)
        else:
            nothing.append(m)

        choices = [(l.pk, l.namn) for l in m.prop]
        if m.lokallag and m.lokallag not in m.prop:
            choices.append((m.lokallag_id, m.lokallag.namn))
        elif not m.lokallag:
            choices.append(('', '(ingen)'))
        m.suggested = SuggestedLokallagForm(
                          auto_id='id_{}_%s'.format(m.pk),
                          initial={
                              'lokallag': m.lokallag_id,
                              'medlem': m.pk,
                          },
                          lokallag_choices=choices,
                          medlem_id=m.pk)

    title = _("Endra lokallag ({c} medlemar)".format(c=queryset.count()))
    opts = modeladmin.model._meta
    app_label = opts.app_label
    context = {
        "title": title,
        "queryset": queryset,
        "opts": opts,
        "app_label": app_label,
        "action": 'suggest_lokallag',
        "action_checkbox_name": helpers.ACTION_CHECKBOX_NAME,
        "already_ok": already_ok,
        "suggested_new": suggested_new,
        "totally_new": totally_new,
    }
    return TemplateResponse(request, "admin/proposed_lokallag.html",
              context, current_app=modeladmin.admin_site.name)
suggest_lokallag.short_description = _(u"Foreslå lokallag")



# -*- coding: utf-8 -*-
# vim: ts=4 sts=4 expandtab ai

# Copyright 2009-2016 Odin Hørthe Omdal

# This file is part of Medlemssys.

# Medlemssys is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# Medlemssys is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with Medlemssys.  If not, see <http://www.gnu.org/licenses/>.

from __future__ import unicode_literals

from django.conf import settings
from django.contrib.auth.models import User
from django.core.urlresolvers import reverse
from django.test import override_settings
from rest_framework import status
from rest_framework.test import APITestCase

from medlemssys.medlem.models import Medlem
from medlemssys.medlem.tests import lagTestMedlemar


class MedlemInnmeldingDefaultSettings(APITestCase):
    def create_data(self, **kw):
        data = dict(token=settings.INNMELDING_TOKEN, postnr='5003', fodt=2000)
        data.update(kw)
        return data

    def test_disallow_read(self):
        r = self.client.get(reverse('api-innmelding'))
        self.assertEquals(status.HTTP_405_METHOD_NOT_ALLOWED, r.status_code)

    def test_require_some_fields(self):
        r = self.client.post(reverse('api-innmelding'), {})
        self.assertEquals(status.HTTP_400_BAD_REQUEST, r.status_code)
        self.assertEquals(r.data.keys(), ['postnr', 'fodt', 'token'])

    def test_create(self):
        r = self.client.post(reverse('api-innmelding'),
            self.create_data(fornamn='Å', etternamn='L'))
        self.assertEquals(status.HTTP_400_BAD_REQUEST, r.status_code)
        # Not really the message we're after, but it works
        self.assertEquals('This field may not be blank.', r.data['token'][0])
        r2 = self.client.post(reverse('api-innmelding'),
            self.create_data(fornamn='Å', etternamn='L', token='x'))
        self.assertEquals(status.HTTP_400_BAD_REQUEST, r2.status_code)
        self.assertEquals(
            'Invalid innmelding token', r2.data['non_field_errors'][0])


@override_settings(INNMELDING_TOKEN='x')
class MedlemInnmelding(MedlemInnmeldingDefaultSettings):
    def test_create(self):
        r = self.client.post(reverse('api-innmelding'),
            self.create_data(fornamn='Å', etternamn='L'))
        self.assertEquals(status.HTTP_201_CREATED, r.status_code)
        self.assertTrue(int(r.data.pop('id')) > 0, 'valid id')
        self.assertEquals(r.data, {
            'fornamn': 'Å', 'etternamn': 'L', 'fodt': 2000, 'postnr': '5003',
            'stad': '?', 'mobnr': '', 'epost': '',
        })
        self.assertEquals(
           Medlem.objects.values_list(
               'etternamn', 'status', 'innmeldingstype')[0],
           ('L', 'I', 'H'))

    def test_namn_not_empty(self):
        r = self.client.post(
            reverse('api-innmelding'), self.create_data(namn=''))
        self.assertEquals(r.data, {
            'non_field_errors': ['Specify either fornamn+etternamn, or namn']})

    def test_cors_nothing_wo_origin(self):
        res = self.client.post(
            reverse('api-innmelding'), self.create_data(namn='x'))
        self.assertEquals(res.has_header('Access-Control-Allow-Origin'), False)

    def test_cors_allowed(self):
        res = self.client.post(
            reverse('api-innmelding'), self.create_data(namn='x'),
            HTTP_ORIGIN='http://example.com:80')
        self.assertEquals(res.has_header('Access-Control-Allow-Origin'), True)
        self.assertEquals(
            res['Access-Control-Allow-Origin'], 'http://example.com:80')

    def test_cors_disallowed(self):
        res = self.client.post(
            reverse('api-innmelding'), self.create_data(namn='x'),
            HTTP_ORIGIN='http://evil.example.com:80')
        self.assertEquals(res.has_header('Access-Control-Allow-Origin'), False)

    def test_namn(self):
        tests = [
          ('Å L', 'Å', 'L'),
          ('Test Foo Bar', 'Test Foo', 'Bar'),
          ('Test Foo Bar Baz', 'Test Foo Bar', 'Baz'),
          ('Test', 'Test', ''),
          ('Test', 'Test', ''),
          #('Test Foo Bar Baz', 'Test Foo', 'Bar Baz'),
        ]
        for full_n, fn, ln in tests:
            r = self.client.post(reverse('api-innmelding'),
                self.create_data(namn=full_n))
            self.assertEquals(
                [r.data[v] for v in ['fornamn', 'etternamn']],
                [fn, ln])

    def test_next_redir(self):
        r = self.client.post(
            reverse('api-innmelding') + '?next=http://example.com/b',
            self.create_data(namn='x'))
        self.assertEquals(r.status_code, 302)
        self.assertEquals(r.content, '')

    def test_next_disallow_redir(self):
        r = self.client.post(
            reverse('api-innmelding') + '?next=http://evil.com/b',
            self.create_data(namn='x'))
        self.assertEquals(r.status_code, 201)


class MedlemAPI(APITestCase):
    def setUp(self):
        self.m = lagTestMedlemar()

    def test_disallow_read(self):
        r = self.client.get('/api/medlem/')
        self.assertEquals(r.status_code, 401)
        r = self.client.post('/api/medlem/', {})
        self.assertEquals(r.status_code, 401)
        r = self.client.get('/api/medlem/1/')
        self.assertEquals(r.status_code, 401)
        r = self.client.put('/api/medlem/1/')
        self.assertEquals(r.status_code, 401)
        r = self.client.delete('/api/medlem/1/')
        self.assertEquals(r.status_code, 401)

    def test_disallow_non_staff(self):
        user = User.objects.create_user(username='user')
        self.client.force_authenticate(user)
        r = self.client.get('/api/medlem/')
        self.assertEquals(r.status_code, 403)
        r = self.client.get('/api/medlem/1/')
        self.assertEquals(r.status_code, 403)

    def test_medlem_list(self):
        user = User.objects.create_user(username='user', is_staff=True)
        self.client.force_authenticate(user)
        r = self.client.get('/api/medlem/')
        self.assertEquals(r.status_code, 200)
        self.assertEquals(r.data['count'], 19)
        self.assertEquals(len(r.data['results']), 19)
        self.assertEquals(
            r.data['results'][0]['fornamn'], '23-betalt-utmeldtnesteaar')

    def test_medlem_single(self):
        user = User.objects.create_user(username='user', is_staff=True)
        self.client.force_authenticate(user)
        m = self.m['25-betalt']
        r = self.client.get('/api/medlem/%d/' % m.pk)
        self.assertEquals(r.status_code, 200)
        self.assertEquals(r.data['fornamn'], '25-betalt')
        self.assertEquals(r.data['etternamn'], 'E')
        self.assertEquals(r.data.keys(), [
             'url', 'fornamn', 'mellomnamn', 'etternamn', 'fodt',
             'postadr', 'ekstraadr', 'postnr', 'epost', 'mobnr',
             'borteadr', 'bortepostnr'])

    def test_medlem_patch(self):
        user = User.objects.create_user(username='user', is_staff=True)
        self.client.force_authenticate(user)
        m = self.m['12']
        r = self.client.patch('/api/medlem/%d/' % m.pk, {'etternamn': 'Test'})
        self.assertEquals(r.status_code, 200)
        self.assertEquals(r.data['fornamn'], '12')
        self.assertEquals(r.data['etternamn'], 'Test')
        self.assertEquals(m.etternamn, 'E')
        m.refresh_from_db()
        self.assertEquals(m.etternamn, 'Test')



from urlparse import urlparse

from django.conf import settings
from django.shortcuts import redirect


def add_cors(func):
    def decorator(request, *args, **kwargs):
        response = func(request, *args, **kwargs)
        origin = request.META.get('HTTP_ORIGIN')
        if (origin and
            _host(origin) in settings.TRUSTED_EXTERNAL_DOMAINS):
            response['Access-Control-Allow-Origin'] = origin
        return response
    return decorator

def add_next_redirect(func):
    def decorator(request, *args, **kwargs):
        next_ = request.GET.get('next')
        response = func(request, *args, **kwargs)
        if (next_ and
            _host(next_) in settings.TRUSTED_EXTERNAL_DOMAINS and
            200 <= response.status_code < 300):
            return redirect(next_)
        return response
    return decorator

def _host(url):
   return urlparse(url).hostname



# -*- coding: utf-8 -*-
# vim: ts=4 sts=4 expandtab ai

# Copyright 2009-2014 Odin Hørthe Omdal

# This file is part of Medlemssys.

# Medlemssys is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# Medlemssys is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with Medlemssys.  If not, see <http://www.gnu.org/licenses/>.
import smtplib

from django.core.mail import EmailMultiAlternatives
from django.db import transaction
from django.template import Context
from django.template import Template

from medlemssys.medlem.models import Giro
from .models import GiroTemplate

@transaction.atomic
def send_ventande_rekningar():
    ventar = Giro.objects.filter(status='V').select_related('medlem')
    for v in ventar:
        if v.betalt():
            v.status = 'F'
        elif not v.medlem.epost or len(v.medlem.epost) < 6:
            v.status = 'E'
        else:
            try:
                send_rekning(v)
                v.status = "1"
            except smtplib.SMTPRecipientsRefused:
                # TODO Do logging
                continue
        v.save()
    return ventar

def send_rekning(giro):
    subject, text_content, html_content = generate_text("medlemspengar",
                                            {"medlem": giro.medlem, "giro": giro})

    msg = EmailMultiAlternatives(subject,
            body=text_content,
            to=[ giro.medlem.epost ])
    msg.attach_alternative(html_content, "text/html")
    msg.send()

def generate_text(trigger, context):
    tmpl = GiroTemplate.objects.get(trigger=trigger)

    subject = Template(tmpl.subject).render(Context(context))
    text_content = Template(tmpl.text_template).render(Context(context))
    html_content = Template(tmpl.html_template).render(Context(context))

    return (subject, text_content, html_content)



# -*- coding: utf-8 -*-
# vim: ts=4 sts=4 expandtab ai

# Copyright 2009-2016 Odin Hørthe Omdal

# This file is part of Medlemssys.

# Medlemssys is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# Medlemssys is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with Medlemssys.  If not, see <http://www.gnu.org/licenses/>.

from importlib import import_module

from django.conf import settings
from django.core.exceptions import ImproperlyConfigured


_cached_class = None

def get_behaviour():
    global _cached_class
    if _cached_class:
        return _cached_class
    mod_name = settings.BEHAVIOUR_MODULE
    mod = import_module(mod_name)
    try:
        _cached_class = getattr(mod, 'Behaviour')
    except AttributeError:
        raise ImproperlyConfigured(
            'Behaviour module "{0}" does not define a '
            '"Behaviour" class'.format(mod_name))
    return _cached_class



# -*- coding: utf-8 -*-
# vim: ts=4 sts=4 expandtab ai
# Copyright 2009-2014 Odin Hørthe Omdal

# This file is part of Medlemssys.

# Medlemssys is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# Medlemssys is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with Medlemssys.  If not, see <http://www.gnu.org/licenses/>.

from django import forms
from django.db.models import Q
from django.shortcuts import render

from medlemssys.medlem.models import Giro
from .views import send_ventande_rekningar

class GiroSearch(forms.Form):
    search_data = forms.CharField(widget=forms.Textarea(
                      attrs={'class': 'vLargeTextField',
                             'rows': 25}))

def send(request):
    if request.method == 'POST':
        sende = send_ventande_rekningar()
        successful = [r for r in sende if r.status == '1']
        unsuccessful = [r for r in sende if r.status != '1']
        return render(request, 'admin/giro/send_done.html', {
                'successful': successful,
                'unsuccessful': unsuccessful })
    ventar = Giro.objects.filter(status='V').select_related('medlem')
    return render(request, 'admin/giro/send.html', {
        'ventar': ventar,
        'title': "Send ventande giroar",
    })

def manual_girosearch(request):
    if request.method == 'POST':
        form = GiroSearch(request.POST)
        form.is_valid()
        giroar = []
        giropk_set = set()
        for line in form.cleaned_data['search_data'].split():
            try:
                line = int(line.strip())
            except:
                giroar.append({'search': line, 'error': "Kunne ikkje gjera om til tal"})
                continue
            giro = Giro.objects.filter(
                Q(kid=line) |
                Q(pk=line) |
                Q(medlem__pk=line))
            if not giro:
                giroar.append({'search': line, 'error': "Fann ikkje giro"})
            else:
                giropk_set.add(str(giro[0].pk))
                giroar.append({'search': line, 'giro': giro[0]})
        admin_url = '/admin/medlem/giro/?id__in={0}'.format(','.join(giropk_set))
        return render(request, 'admin/manual_girosearch.html', {
            'admin_url': admin_url,
            'form': form,
            'giroar': giroar,
            'title': u"Manuelt girosøk",
        })
    else:
        form = GiroSearch()
    return render(request, 'admin/manual_girosearch.html', {
        'form': form,
        'title': "Manuelt girosøk",
    })



# -*- coding: utf-8 -*-
# vim: ts=4 sts=4 expandtab ai

# Copyright 2009-2016 Odin Hørthe Omdal

# This file is part of Medlemssys.

# Medlemssys is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# Medlemssys is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with Medlemssys.  If not, see <http://www.gnu.org/licenses/>.

from __future__ import unicode_literals

from django.conf import settings
from django.views.decorators.csrf import csrf_exempt
from rest_framework import status
from rest_framework import viewsets
from rest_framework.decorators import api_view
from rest_framework.decorators import permission_classes
from rest_framework.permissions import AllowAny
from rest_framework.response import Response

from medlemssys.api import serializers
from medlemssys.medlem import models as medlem_models
from medlemssys.utils import add_cors
from medlemssys.utils import add_next_redirect


@csrf_exempt
@add_cors
@add_next_redirect
@api_view(['POST'])
@permission_classes((AllowAny,))
def medlem_innmelding(request):
    serializer = serializers.InnmeldingMedlemSerializer(data=request.data)
    if serializer.is_valid():
        serializer.save()
        return Response(serializer.data, status=status.HTTP_201_CREATED)
    return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)


class MedlemViewSet(viewsets.ModelViewSet):
    queryset = medlem_models.Medlem.objects.all()
    serializer_class = serializers.MedlemSerializer
